package Stage2;

import java.util.ArrayList;
import javafx.scene.Group;
import javafx.scene.shape.*;
import javafx.util.Duration;
import javafx.animation.*;
import java.awt.geom.*;

public class Train extends Group {

    private ArrayList<Station> stations;
    private int stationNumber;
    private int direction;
    private Path path;
    private PathTransition pt;
    
    public Train(ArrayList<Station> stations){
        this(stations, 0, 1);        
    }

    public Train(ArrayList<Station> stations, int startingStation, int direction){
        this.stations = stations;
        stationNumber = startingStation;
        this.direction = direction;
        path = new Path();
        pt = new PathTransition();

        Polygon trainShape = new Polygon(0,0, 0,13, 23,13, 23,0);
        this.getChildren().add(trainShape);

        Station actualStation = stations.get(stationNumber);
        Station nextStation = stations.get(stationNumber+direction);

        path.getElements().add(new MoveTo(actualStation.getX(),actualStation.getY()));
        path.getElements().add(new LineTo(nextStation.getX(),nextStation.getY()));

        stationNumber+=direction;

        double distance = Point2D.distance(actualStation.getX(),actualStation.getY(),nextStation.getX(),nextStation.getY());
        pt.setDuration(new Duration(distance*20));
        pt.setNode(trainShape);
        pt.setPath(path);
        pt.setOrientation(PathTransition.OrientationType.ORTHOGONAL_TO_TANGENT);
        pt.play();

        pt.setOnFinished(e ->changePath());
    }

    private void changePath(){

        if(direction == 1 && stationNumber == stations.size() - 1){
            direction = -1;
        }
        else if(direction == -1 && stationNumber == 0){
            direction = 1;
        }

        Station actualStation = stations.get(stationNumber);
        Station nextStation = stations.get(stationNumber+direction);

        double distance = Point2D.distance(actualStation.getX(),actualStation.getY(),nextStation.getX(),nextStation.getY());
        
        pt.setDuration(new Duration(distance*20));

        path.getElements().clear();
        path.getElements().add(new MoveTo(actualStation.getX(),actualStation.getY()));
        path.getElements().add(new LineTo(nextStation.getX(),nextStation.getY()));
        stationNumber+=direction;
        
        pt.setDelay(new Duration(1000));
        pt.play();
        
    }
}