package Stage3;

import java.util.ArrayList;
import javafx.scene.Group;
import javafx.scene.input.*;
import javafx.scene.layout.Pane;
import java.util.Random;

public class MapView extends Pane {
    private Group map;

    private double mouseX;
    private double mouseY;

    public MapView(ArrayList<Line> lines, ArrayList<Station> stations){
        map = new Group();
        this.setStyle("-fx-background-color: transparent");
        // Draw the lines
        for (Line line : lines) {
            map.getChildren().add(line.getView());
        }

        // Draw the stations
        for (Station station : stations) {
            map.getChildren().add(station.getView());
        }

        // Add trains
        Random rand = new Random();
        for (Line line : lines) {
            // Format
            // line.addTrain(startingStation, direction (1 or -1));
            line.addTrain(rand.nextInt(line.getStations().size()-1),1);
            line.addTrain(rand.nextInt(line.getStations().size()-1)+1,-1);
        }

        this.getChildren().add(map);
        this.setOnScroll(e->scroll(e));
        this.setOnMousePressed(e->saveMouseCoordinates(e));
        this.setOnMouseDragged(e->dragMap(e));
    }

    private void scroll(ScrollEvent e){
        double zoomFactor = 1.05;
        if (e.getDeltaY() < 0){
            zoomFactor = 2.0 - zoomFactor;
        }
        map.setScaleX(map.getScaleX()*zoomFactor);
        map.setScaleY(map.getScaleY()*zoomFactor);
    }

    private void saveMouseCoordinates(MouseEvent e){
        this.mouseX = e.getX();
        this.mouseY = e.getY();
    }
    private void dragMap(MouseEvent e){

        this.setTranslateX(this.getTranslateX() + e.getX() - mouseX);
        this.setTranslateY(this.getTranslateY() + e.getY() - mouseY);
        
    }
}