package Stage3;

import java.util.ArrayList;
import javafx.collections.*;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
//import javafx.scene.paint.Color;

public class TabMenu extends TabPane{
    // FIELDS
    private static BorderPane stationPane;
    private static BorderPane trainPane;
    private static BorderPane infoBP;
    private static GridPane routePane;
    private static Train train;
    private static Station station;

    public TabMenu(ArrayList<Line> lines,ArrayList<Station> stations){
        Tab stationTab = new Tab();
        stationPane = new BorderPane();
        stationTab.setText("Estación");
        stationTab.setClosable(false);
        stationTab.setContent(stationPane);
        
        Tab trainTab = new Tab();
        trainPane = new BorderPane();
        trainTab.setText("Tren");
        trainTab.setClosable(false);
        trainTab.setContent(trainPane);
        
        Tab infoTab = new Tab();
        infoBP = createInfoTab(lines);
        infoTab.setText("Información");
        infoTab.setClosable(false);
        infoTab.setContent(infoBP);

        Tab routeTab = new Tab();
        routeTab.setText("Buscador de Rutas");
        routeTab.setClosable(false);
        TextArea routeT = new TextArea();
        routeT.setText("WORK IN PROGRESS... \n\n\n" + WORKINPROGRESS());
        routePane = new GridPane();
        try {
            routePane = FXMLLoader.<GridPane>load(getClass().getResource("grid.fxml"));
        } catch (Exception e) {
            System.out.println("wrong.");
        }

        ArrayList<String> list = new ArrayList<String>();

        // Se agregan las estaciones
        for(Station i: stations){
            list.add(i.stationName);
        }
        ObservableList<String> options = FXCollections.observableList(list);

        final ComboBox origComboBox = new ComboBox(options);
        final ComboBox destComboBox = new ComboBox(options);
        origComboBox.setValue("-- Origen --");
        destComboBox.setValue("-- Destino --");
        // orden - node, col, row, spanC, spanR
        routePane.add(origComboBox, 0, 2, 2, 1);
        routePane.add(destComboBox, 0, 4, 2, 1);

        routePane.add(routeT, 0,6, 3,1);

        routeTab.setContent(routePane);
        
        this.getTabs().addAll(infoTab, stationTab, trainTab, routeTab);
        this.setMinWidth(400);
        this.setPrefWidth(600);
        this.setStyle("-fx-background-color: gainsboro; -fx-border-color:dimgray");
        
    }

    public static void changeStationInfo(Station _station, boolean newStation){
        // If it is a different station
        if(newStation){
            station = _station;
        }

        if(station == null){
            return;
        }

        stationPane.getChildren().clear();
        String dataS;
        TextArea stationText = new TextArea();
        stationText.setMaxWidth(400);     stationText.setMaxHeight(400);
        stationText.setWrapText(true);
        
        dataS = "Estación : " + station.stationName + "\n";
        
        ArrayList<Line> lines = station.getLine();

        for (Line line : lines) {
            dataS = dataS + "\n---Línea : " + line.lineName + "---\n";
            String firstStation = line.getStations().get(0).stationName;
            String lastStation = line.getStations().get(line.getStations().size()-1).stationName;

            int stationIndex = line.getStations().indexOf(station);
            double bestTime = Double.POSITIVE_INFINITY;
            
            //First Station
            if(stationIndex == 0){
                bestTime = bestTime(line.getTrains(),stationIndex, -1);
                if(Double.isInfinite(bestTime)){
                    dataS = dataS + "No hay trenes con destino " + lastStation + "\n";
                }
                else{
                    bestTime -= AppTime.getCurrentTime();
                    dataS = dataS + "Siguiente tren con destino " + lastStation + "\n";
                    dataS = dataS + "En " + (int)bestTime + "[s]" + "\n";
                }
            }
            //Last Station
            else if (stationIndex == line.getStations().size()-1){
                bestTime = bestTime(line.getTrains(),stationIndex, 1);
                if(Double.isInfinite(bestTime)){
                    dataS = dataS + "No hay trenes con destino " + firstStation + "\n";
                }
                else{
                    bestTime -= AppTime.getCurrentTime();
                    dataS = dataS + "Siguiente tren con destino " + firstStation + "\n";
                    dataS = dataS + "En " + (int)bestTime + "[s]" + "\n";
                }
            }
            //Any other station
            else{
                bestTime = bestTime(line.getTrains(),stationIndex, 1);
                if(Double.isInfinite(bestTime)){
                    dataS = dataS + "No hay trenes con destino " + lastStation + "\n\n";
                }
                else{
                    bestTime -= AppTime.getCurrentTime();
                    dataS = dataS + "Siguiente tren con destino " + lastStation + "\n";
                    dataS = dataS + "En " + (int)bestTime + "[s]" + "\n";
                }

                bestTime = bestTime(line.getTrains(),stationIndex, -1);
                if(Double.isInfinite(bestTime)){
                    dataS = dataS + "No hay trenes con destino " + firstStation + "\n";
                }
                else{
                    bestTime -= AppTime.getCurrentTime();
                    dataS = dataS + "Siguiente tren con destino " + firstStation + "\n";
                    dataS = dataS + "En " + (int)bestTime + "[s]" + "\n";
                }
            }
        }            
        stationText.setText(dataS);
        stationPane.setCenter(stationText);
    }

    private static double bestTime(ArrayList<Train> trains,int stationIndex, int direction){
        double bestTime = Double.POSITIVE_INFINITY;
        double time;
        for (Train train : trains) {
            time = train.time2Destination(stationIndex,direction);
            bestTime = (time < bestTime) ? time : bestTime;
        }
        return bestTime;
    }
    
    public static void changeTrainInfo(Train _train, boolean newTrain){
        // If it is a different train
        if (newTrain){
            train = _train;
        }

        if(train == null){
            return;
        }
        
        trainPane.getChildren().clear();
        String dataT = "";
        TextArea trainText = new TextArea();
        trainText.setMaxWidth(400);    trainText.setMaxHeight(400);
        trainText.setWrapText(true);
        
        if(train.getDirection() == 1){
            String finalStation = train.getLine().getStations().get(train.getLine().getStations().size()-1).stationName;
            dataT = dataT + "Tren de la Linea " + train.getLine().lineName + ", con destino " + finalStation + "\n";
        }
        else{
            String firstStation = train.getLine().getStations().get(0).stationName;
            dataT = dataT + "Tren de la Linea " + train.getLine().lineName + ", con destino " + firstStation + "\n";
        }
        dataT = dataT + "Siguiente estacion : " + train.getStation().stationName + "\n";
        int time = (int)train.time2Destination()- (int)AppTime.getCurrentTime();
        dataT = dataT + "Tren con " + train.getPassengers() + " pasajeros a bordo" + "\n";
        dataT = dataT + "El tren llegara en : " + time + "[s]" + "\n";
        
        trainText.setText(dataT);
        trainPane.setCenter(trainText);
    }   
    /**
     * Sets the information tab.
     */ 
    public static BorderPane createInfoTab(ArrayList<Line> lines){
        infoBP =  new BorderPane();
        MenuBar menuBar =  new MenuBar();
        Menu mLine = new Menu("Líneas");
        MenuItem item;
        String lineName;
        TextArea infoT = new TextArea();
        infoT.setText("\n\n\nGracias por usar nuestro servicio");
        infoT.setMaxWidth(400);     infoT.setMaxHeight(400);
        infoT.setWrapText(true);

        // Actions
        for(Line i:lines){
            lineName = i.lineName;
            item = new MenuItem("--- Línea " + lineName + " ---");
            mLine.getItems().add(item);
            item.setOnAction(e -> {
                infoT.setText(i.printStations());
            });
        }
        item = new MenuItem("--- Info ---");
        mLine.getItems().add(item);
        item.setOnAction(e -> {
            infoT.setText("\n\n\nGracias por usar nuestro servicio");
        });
        menuBar.getMenus().addAll(mLine);
        infoBP.setTop(menuBar);
        infoBP.setCenter(infoT);

        return infoBP;
    }

    private String WORKINPROGRESS(){
        String data;
        data =  "                                               ///////////////////////\n";
        data += "                                            ,/             /          /\n";
        data += "                                           //////*        /    /////  /* \n";
        data += "                                         ,/      /         /   ///////  //   \n";
        data += "                                        //       /         /     //(      /. \n";
        data += "                                      ./        ,/*       ,//     /        //\n";
        data += "                                     /(       (/////(   ,/////(  /           /\n";
        data += "             //////////////////////   //       /////     (////   /          /, \n";
        data += "           .///////////.  */////////.  ,/                         /       // \n";
        data += "          ///////////*      //////////   /*                    //,       /*\n";
        data += "         /////////////     .///////////.  /(  (//////////////(/        ,/ \n";
        data += "       ///////////// .///////////////////   //                        //\n";
        data += "      ///////////. ,*/////   .///////////    /(                     ,*\n";
        data += "    ///////////  ///////                .///   //////////////////////\n";
        data += "               //////////     ///////////////\n";
        data += "    *//////////////////////////////////////,   //////////////////////, \n";
        data += "      *///////////      /////////////////*   /*//////////////////////// \n";
        data += "       *//////////                                 ///////////.   //////. \n";
        data += "         //////////,  ,////////////////   //////////                ////// \n";
        data += "          ////////////////////////////   /////  ./////////////     ////////  \n";
        data += "            ////////////////////////   /////            ,//////////////////// \n";
        data += "             //////////////////////   //////,     ///////  /////////////      \n";
        data += "                                      /////////////////////             //////*\n";
        data += "                                       //////////////////////////////////////\n";
        data += "                                        ,/////////////////////*\n";
        data += "                                          //////////////////  *//////////*\n";
        data += "                                           *//////////.      ////////////\n";
        data += "                                             *////////,      //////////\n";
        data += "                                              //////////////////////// \n";
        return data;
    }
}