package Stage4;

import java.util.ArrayList;
import javafx.scene.Group;
import javafx.scene.input.*;
import javafx.scene.layout.BorderPane;
import java.util.Random;
import javafx.scene.shape.Rectangle;
import javafx.scene.paint.Color;

public class MapView extends BorderPane {
    private Group map;
    private double mouseX;
    private double mouseY;

    public MapView(ArrayList<Line> lines, ArrayList<Station> stations){

        map = new Group();
        Rectangle mapBack = new Rectangle(0,0,10,10);
        mapBack.setFill(Color.ANTIQUEWHITE);
        map.getChildren().add(mapBack);
        // Draw the lines
        for (Line line : lines) {
            map.getChildren().add(line.getView());
        }

        // Draw the stations
        for (Station station : stations) {
            map.getChildren().add(station.getView());
        }

        // Add trains
        Random rand = new Random();
        for (Line line : lines) {
            // Format
            // line.addTrain(startingStation, direction (1 or -1));
            line.addTrain(rand.nextInt(line.getStations().size()-1),1);
            line.addTrain(rand.nextInt(line.getStations().size()-1)+1,-1);
        }
        mapBack.setWidth(map.maxWidth(500)+25);
        mapBack.setHeight(map.maxHeight(500)+25);

        this.setCenter(map);;
        this.setOnScroll(e->scroll(e));
        this.setOnMousePressed(e->saveMouseCoordinates(e));
        this.setOnMouseDragged(e->dragMap(e));
    }

    private void scroll(ScrollEvent e){
        double zoomFactor = 1.05;
        if (e.getDeltaY() < 0){
            zoomFactor = 2.0 - zoomFactor;
        }
        map.setScaleX(map.getScaleX()*zoomFactor);
        map.setScaleY(map.getScaleY()*zoomFactor);
    }

    private void saveMouseCoordinates(MouseEvent e){
        this.mouseX = e.getX();
        this.mouseY = e.getY();
    }
    private void dragMap(MouseEvent e){

        this.setTranslateX(this.getTranslateX() + e.getX() - mouseX);
        this.setTranslateY(this.getTranslateY() + e.getY() - mouseY);
        
    }
}