package Stage4;

import javafx.scene.Group;
import javafx.scene.shape.*;
import javafx.util.Duration;
import java.util.Random;
import javafx.animation.*;
import java.awt.geom.*;
import javafx.animation.Timeline;
import javafx.animation.KeyFrame;

public class Train extends Group {

    private Line line;
    private int stationNumber;
    private int direction;
    private Path path;
    private PathTransition pt;
    private float DepartureTime;
    private float ETA;
    private Timeline animation;

    private int passengers;

    private static int MAX;
    private static float TRAIN_SPEED;

    static{
        TRAIN_SPEED = 0.016f;
        MAX = 100;
    }
    
    public Train(Line line){
        this(line, 0, 1);        
    }

    public Train(Line line, int startingStation, int direction){
        Random rand = new Random();
        passengers = rand.nextInt(MAX);
        this.line = line;
        stationNumber = startingStation;
        this.direction = direction;
        path = new Path();
        pt = new PathTransition();

        Polygon trainShape = new Polygon(0,0, 0,15, 32,15, 32,0);
        this.getChildren().add(trainShape);

        Station actualStation = line.getStations().get(stationNumber);
        Station nextStation = line.getStations().get(stationNumber+direction);

        path.getElements().add(new MoveTo(actualStation.getX(),actualStation.getY()));
        path.getElements().add(new LineTo(nextStation.getX(),nextStation.getY()));

        stationNumber+=direction;

        double distance = Point2D.distance(actualStation.getX(),actualStation.getY(),nextStation.getX(),nextStation.getY());
        pt.setDuration(new Duration(distance/TRAIN_SPEED));
        pt.setNode(trainShape);
        pt.setPath(path);
        pt.setOrientation(PathTransition.OrientationType.ORTHOGONAL_TO_TANGENT);
        pt.play();

        DepartureTime = AppTime.getCurrentTime();
        ETA = DepartureTime + ((float)distance/TRAIN_SPEED)/1000 + 1;
        
        pt.setOnFinished(e ->{
            changePath();
        });

        trainShape.setOnMouseClicked(e->{
            TabMenu.changeTrainInfo(this,true);
        });

        animation = new Timeline(new KeyFrame(Duration.millis(1000), e->TabMenu.changeTrainInfo(this,false)));
        animation.setCycleCount(Timeline.INDEFINITE);
        animation.play();

    }

    public Line getLine(){
        return line;
    }

    public Station getStation(){
        return line.getStations().get(stationNumber);
    }

    public int getDirection(){
        return direction;
    }

    public Station getNextStation(){
        return line.getStations().get(stationNumber+direction);
    }

    public int getPassengers(){
        return passengers;
    }

    public float time2Destination(){
        return ETA;
    }

    public double time2Destination(int Destination, int direction){

        if(this.direction != direction){
            return Double.POSITIVE_INFINITY;
        }

        if(stationNumber < Destination && direction == -1){
            return Double.POSITIVE_INFINITY;
        }
        else if(stationNumber > Destination && direction == 1){
            return Double.POSITIVE_INFINITY;
        }
        float time = ETA;
        if(direction == 1){
            for(int i = stationNumber; i < Destination; i++){
                Station actualStation = line.getStations().get(i);
                Station nextStation = line.getStations().get(i+1);
                time+= 1 + (Point2D.distance(actualStation.getX(),actualStation.getY(),nextStation.getX(),nextStation.getY())/TRAIN_SPEED)/1000;
            }
        }
        
        if(direction == -1){
            for(int i = stationNumber; i > Destination; i--){
                Station actualStation = line.getStations().get(i);
                Station nextStation = line.getStations().get(i-1);
                time+= 1 + (Point2D.distance(actualStation.getX(),actualStation.getY(),nextStation.getX(),nextStation.getY())/TRAIN_SPEED)/1000;
            }
        }   
        return time;
    }

    private void changePath(){

        Random rand = new Random();

        passengers += rand.nextInt(40) - 20;
        passengers = (passengers >= MAX) ? MAX : ((passengers < 0) ? 0 : passengers);


        if(direction == 1 && stationNumber == line.getStations().size() - 1){
            direction = -1;
        }
        else if(direction == -1 && stationNumber == 0){
            direction = 1;
        }

        Station actualStation = line.getStations().get(stationNumber);
        Station nextStation = line.getStations().get(stationNumber+direction);

        double distance = Point2D.distance(actualStation.getX(),actualStation.getY(),nextStation.getX(),nextStation.getY());
        
        pt.setDuration(new Duration(distance/TRAIN_SPEED));

        path.getElements().clear();
        path.getElements().add(new MoveTo(actualStation.getX(),actualStation.getY()));
        path.getElements().add(new LineTo(nextStation.getX(),nextStation.getY()));
        
        pt.setDelay(new Duration(1000));
        
        DepartureTime = AppTime.getCurrentTime();
        ETA = DepartureTime + ((float)distance/TRAIN_SPEED)/1000 + 1;
        
        stationNumber+=direction;
        
        pt.play();
    }
}